@@ -87,11 +87,11 @@ available at the moment:
 `OQ_DISTRIBUTE` set tp "ipython"
    use the ipyparallel concurrency mechanism (experimental)
 
-There is no such a thing as OQ_DISTRIBUTE="threading"; it would be trivial
-to do, but the performance of using threads instead of processes is terrible
-for the kind of applications we are interested in (CPU-dominated, which large
-tasks such that the time to spawn a new process is negligible with respect
-to the time to perform the task).
+There is also an `OQ_DISTRIBUTE` = "threadpool"; however the
+performance of using threads instead of processes is normally bad for the
+kind of applications we are interested in (CPU-dominated, which large
+tasks such that the time to spawn a new process is negligible with
+respect to the time to perform the task), so it is not recommended.
 
 The Starmap.apply API
 ====================================
@@ -133,6 +133,7 @@ having finished all the short tasks, but you have to wait for days for
 the single core processing the slow task). The OpenQuake engine does
 a great deal of work trying to split slow sources in more manageable
 fast sources.
+
 """
 from __future__ import print_function
 import os
@@ -143,14 +144,22 @@ import socket
 import inspect
 import logging
 import operator
-import traceback
 import functools
 import subprocess
 import multiprocessing.dummy
 from multiprocessing.connection import Client, Listener
-from concurrent.futures import as_completed, ProcessPoolExecutor, Future
+from concurrent.futures import (
+    as_completed, ThreadPoolExecutor, ProcessPoolExecutor, Future)
+
 import numpy
-from openquake.baselib import hdf5
+try:
+    from setproctitle import setproctitle
+except ImportError:
+    def setproctitle(title):
+        "Do nothing"
+
+from openquake.baselib import hdf5, config
+from openquake.baselib.workerpool import safely_call, _starmap
 from openquake.baselib.python3compat import pickle
 from openquake.baselib.performance import Monitor, virtual_memory
 from openquake.baselib.general import (
@@ -158,10 +167,10 @@ from openquake.baselib.general import (
 
 executor = ProcessPoolExecutor()
 executor.pids = ()  # set by wakeup_pool
-# the num_tasks_hint is chosen to be 5 times bigger than the name of
+# the num_tasks_hint is chosen to be 3 times bigger than the name of
 # cores; it is a heuristic number to get a good distribution;
 # it has no more significance than that
-executor.num_tasks_hint = executor._max_workers * 5
+executor.num_tasks_hint = executor._max_workers * 3
 
 OQ_DISTRIBUTE = os.environ.get('OQ_DISTRIBUTE', 'futures').lower()
 
@@ -178,28 +187,26 @@ elif OQ_DISTRIBUTE == 'ipython':
 
 def oq_distribute(task=None):
     """
-    If the task has an attribute `shared_dir_on` which is false,
-    return 'futures' even if OQ_DISTRIBUTE is `celery`, otherwise
-    return the current value of the variable OQ_DISTRIBUTE;
-    if undefined, return 'futures'.
+    :returns: the value of OQ_DISTRIBUTE or 'futures'
     """
-    env = os.environ.get('OQ_DISTRIBUTE', 'futures').lower()
-    if hasattr(task, 'shared_dir_on'):
-        if env == 'celery' and not task.shared_dir_on():
-            logging.warn(
-                'Task `%s` will be run on the controller node only, since '
-                'no `shared_dir` has been specified' % task.__name__)
-            return 'futures'
-    return env
+    dist = os.environ.get('OQ_DISTRIBUTE', 'futures').lower()
+    read_access = getattr(task, 'read_access', True)
+    if dist == 'celery' and not read_access:
+        raise ValueError('You must configure the shared_dir in openquake.cfg '
+                         'in order to be able to run %s with celery' %
+                         task.__name__)
+    return dist
 
 
 def check_mem_usage(monitor=Monitor(),
-                    soft_percent=90, hard_percent=100):
+                    soft_percent=None, hard_percent=None):
     """
     Display a warning if we are running out of memory
 
     :param int mem_percent: the memory limit as a percentage
     """
+    soft_percent = soft_percent or config.memory.soft_mem_limit
+    hard_percent = hard_percent or config.memory.hard_mem_limit
     used_mem_percent = virtual_memory().percent
     if used_mem_percent > hard_percent:
         raise MemoryError('Using more memory than allowed by configuration '
@@ -207,58 +214,10 @@ def check_mem_usage(monitor=Monitor(),
                           (used_mem_percent, hard_percent))
     elif used_mem_percent > soft_percent:
         hostname = socket.gethostname()
-        monitor.send('warn', 'Using over %d%% of the memory in %s!',
+        logging.warn('Using over %d%% of the memory in %s!',
                      used_mem_percent, hostname)
 
 
-def safely_call(func, args, pickle=False, conn=None):
-    """
-    Call the given function with the given arguments safely, i.e.
-    by trapping the exceptions. Return a pair (result, exc_type)
-    where exc_type is None if no exceptions occur, otherwise it
-    is the exception class and the result is a string containing
-    error message and traceback.
-
-    :param func: the function to call
-    :param args: the arguments
-    :param pickle:
-        if set, the input arguments are unpickled and the return value
-        is pickled; otherwise they are left unchanged
-    """
-    with Monitor('total ' + func.__name__, measuremem=True) as child:
-        if pickle:  # measure the unpickling time too
-            args = [a.unpickle() for a in args]
-        if args and isinstance(args[-1], Monitor):
-            mon = args[-1]
-            mon.children.append(child)  # child is a child of mon
-            child.hdf5path = mon.hdf5path
-        else:
-            mon = child
-        check_mem_usage(mon)  # check if too much memory is used
-        mon.flush = NoFlush(mon, func.__name__)
-        try:
-            got = func(*args)
-            if inspect.isgenerator(got):
-                got = list(got)
-            res = got, None, mon
-        except:
-            etype, exc, tb = sys.exc_info()
-            tb_str = ''.join(traceback.format_tb(tb))
-            res = ('\n%s%s: %s' % (tb_str, etype.__name__, exc), etype, mon)
-
-        # NB: flush must not be called in the workers - they must not
-        # have access to the datastore - so we remove it
-        rec_delattr(mon, 'flush')
-
-    if pickle:  # it is impossible to measure the pickling time :-(
-        res = Pickled(res)
-    if conn:  # send the result via the connection
-        conn.send(res)
-        conn.close()
-        return None
-    return res
-
-
 def mkfuture(result):
     fut = Future()
     fut.set_result(result)
@@ -278,7 +237,10 @@ class Pickled(object):
     def __init__(self, obj):
         self.clsname = obj.__class__.__name__
         self.calc_id = str(getattr(obj, 'calc_id', ''))  # for monitors
-        self.pik = pickle.dumps(obj, pickle.HIGHEST_PROTOCOL)
+        try:
+            self.pik = pickle.dumps(obj, pickle.HIGHEST_PROTOCOL)
+        except TypeError as exc:  # can't pickle, show the obj in the message
+            raise TypeError('%s: %s' % (exc, obj))
 
     def __repr__(self):
         """String representation of the pickled object"""
@@ -345,16 +307,18 @@ class IterResult(object):
     :param taskname:
         the name of the task
     :param num_tasks:
-        the total number of expected futures (None if unknown)
+        the total number of expected futures
     :param progress:
         a logging function for the progress report
+    :param sent:
+        the number of bytes sent (0 if OQ_DISTRIBUTE=no)
     """
     task_data_dt = numpy.dtype(
         [('taskno', numpy.uint32), ('weight', numpy.float32),
          ('duration', numpy.float32)])
 
-    def __init__(self, futures, taskname, num_tasks=None,
-                 progress=logging.info):
+    def __init__(self, futures, taskname, num_tasks,
+                 progress=logging.info, sent=0):
         self.futures = futures
         self.name = taskname
         self.num_tasks = num_tasks
@@ -362,11 +326,14 @@ class IterResult(object):
             self.progress = logging.debug
         else:
             self.progress = progress
-        self.sent = 0  # set in Starmap.submit_all
+        self.sent = sent
         self.received = []
         if self.num_tasks:
             self.log_percent = self._log_percent()
             next(self.log_percent)
+        if sent:
+            self.progress('Sent %s of data in %s task(s)',
+                          humansize(sum(sent.values())), num_tasks)
 
     def _log_percent(self):
         yield 0
@@ -505,9 +472,18 @@ class Starmap(object):
         self.task_func = oqtask
         self.task_args = task_args
         self.name = name or oqtask.__name__
+        self.init(oqtask)
         self.results = []
-        self.sent = AccumDict()
         self.distribute = oq_distribute(oqtask)
+        if self.distribute == 'threadpool':
+            self.executor = ThreadPoolExecutor(executor.num_tasks_hint)
+        if self.distribute == 'ipython' and isinstance(
+                self.executor, ProcessPoolExecutor):
+            client = ipp.Client()
+            self.__class__.executor = client.executor()
+
+    def init(self, oqtask):
+        self.sent = AccumDict()
         # a task can be a function, a class or an instance with a __call__
         if inspect.isfunction(oqtask):
             self.argnames = inspect.getargspec(oqtask).args
@@ -515,10 +491,6 @@ class Starmap(object):
             self.argnames = inspect.getargspec(oqtask.__init__).args[1:]
         else:  # instance with a __call__ method
             self.argnames = inspect.getargspec(oqtask.__call__).args[1:]
-        if self.distribute == 'ipython' and isinstance(
-                self.executor, ProcessPoolExecutor):
-            client = ipp.Client()
-            self.__class__.executor = client.executor()
 
     def progress(self, *args):
         """
@@ -539,24 +511,19 @@ class Starmap(object):
         check_mem_usage()
         # log a warning if too much memory is used
         if self.distribute == 'no':
-            sent = {}
             res = safely_call(self.task_func, args)
         else:
-            piks = pickle_sequence(args)
-            sent = {arg: len(p) for arg, p in zip(self.argnames, piks)}
-            res = self._submit(piks)
-        self.sent += sent
+            res = self._submit(args)
         self.results.append(res)
-        return sent
 
     def _submit(self, piks):
         if self.distribute == 'celery':
-            res = safe_task.delay(self.task_func, piks, True)
+            res = safe_task.delay(self.task_func, piks)
             self.task_ids.append(res.task_id)
             return res
         else:  # submit tasks by using the ProcessPoolExecutor or ipyparallel
             return self.executor.submit(
-                safely_call, self.task_func, piks, True)
+                safely_call, self.task_func, piks)
 
     def _iterfutures(self):
         # compatibility wrapper for different concurrency frameworks
@@ -589,7 +556,8 @@ class Starmap(object):
         :param acc: the initial value of the accumulator
         :returns: the final value of the accumulator
         """
-        acc = self.submit_all().reduce(agg, acc or AccumDict())
+        acc = AccumDict() if acc is None else acc
+        acc = self.submit_all().reduce(agg, acc)
         self.results = []
         return acc
 
@@ -601,51 +569,76 @@ class Starmap(object):
         """
         return self.reduce(self, lambda acc, res: acc + 1, 0)
 
+    @property
+    def num_tasks(self):
+        """
+        The number of tasks, if known, or the empty string otherwise.
+        """
+        try:
+            return len(self.task_args)
+        except TypeError:  # generators have no len
+            return ''
+        # NB: returning -1 breaks openquake.hazardlib.tests.calc.
+        # hazard_curve_new_test.HazardCurvesTestCase02 :-(
+
     def submit_all(self):
         """
         :returns: an IterResult object
         """
-        try:
-            nargs = len(self.task_args)
-        except TypeError:  # generators have no len
-            nargs = ''
-        if nargs == 1:
-            [args] = self.task_args
+        if self.num_tasks == 1:
+            [args] = self.add_task_no(self.task_args, pickle=False)
             self.progress('Executing "%s" in process', self.name)
             fut = mkfuture(safely_call(self.task_func, args))
-            return IterResult([fut], self.name, nargs)
+            return IterResult([fut], self.name, self.num_tasks)
 
-        if self.distribute == 'qsub':
-            logging.warn('EXPERIMENTAL: sending tasks to the grid engine')
-            allargs = list(self.task_args)
+        elif self.distribute == 'zmq':  # experimental
+            allargs = self.add_task_no(self.task_args)
+            w = config.zworkers
+            it = _starmap(
+                self.task_func, allargs,
+                w.master_host, w.task_in_port, w.receiver_ports)
+            ntasks = next(it)
+            return IterResult(it, self.name, ntasks, self.progress, self.sent)
+
+        elif self.distribute == 'qsub':  # experimental
+            allargs = list(self.add_task_no(self.task_args, pickle=False))
+            logging.warn('Sending %d tasks to the grid engine', len(allargs))
             return IterResult(qsub(self.task_func, allargs),
-                              self.name, len(allargs), self.progress)
+                              self.name, len(allargs),
+                              self.progress, self.sent)
 
         task_no = 0
-        for args in self.task_args:
+        for args in self.add_task_no(self.task_args):
             task_no += 1
-            if task_no == 1:  # first time
-                self.progress('Submitting %s "%s" tasks', nargs, self.name)
-            if isinstance(args[-1], Monitor):
-                # add incremental task number and task weight
-                args[-1].task_no = task_no
-                args[-1].weight = getattr(args[0], 'weight', 1.)
             self.submit(*args)
         if not task_no:
             self.progress('No %s tasks were submitted', self.name)
         # NB: keep self._iterfutures() an iterator, especially with celery!
         ir = IterResult(self._iterfutures(), self.name, task_no,
-                        self.progress)
-        ir.sent = self.sent  # for information purposes
-        if self.sent:
-            self.progress('Sent %s of data in %d task(s)',
-                          humansize(sum(self.sent.values())),
-                          ir.num_tasks)
+                        self.progress, self.sent)
         return ir
 
     def __iter__(self):
         return iter(self.submit_all())
 
+    def add_task_no(self, iterargs, pickle=True):
+        """
+        Add .task_no and .weight to the monitor and yield back
+        the arguments by pickling them if pickle is True.
+        """
+        for task_no, args in enumerate(iterargs, 1):
+            if isinstance(args[-1], Monitor):
+                # add incremental task number and task weight
+                args[-1].task_no = task_no
+                args[-1].weight = getattr(args[0], 'weight', 1.)
+            if pickle:
+                args = pickle_sequence(args)
+                self.sent += {a: len(p) for a, p in zip(self.argnames, args)}
+            if task_no == 1:  # first time
+                self.progress('Submitting %s "%s" tasks', self.num_tasks,
+                              self.name)
+            yield args
+
 
 def do_not_aggregate(acc, value):
     """
@@ -658,33 +651,13 @@ def do_not_aggregate(acc, value):
     return acc
 
 
-class NoFlush(object):
-    # this is instantiated by safely_call
-    def __init__(self, monitor, taskname):
-        self.monitor = monitor
-        self.taskname = taskname
-
-    def __call__(self):
-        raise RuntimeError('Monitor(%r).flush() must not be called '
-                           'by %s!' % (self.monitor.operation, self.taskname))
-
-
-def rec_delattr(mon, name):
-    """
-    Delete attribute from a monitor recursively
-    """
-    for child in mon.children:
-        rec_delattr(child, name)
-    if name in vars(mon):
-        delattr(mon, name)
-
-
 if OQ_DISTRIBUTE == 'celery':
     safe_task = task(safely_call,  queue='celery')
 
 
 def _wakeup(sec):
-    """Waiting functions, used to wake up the process pool"""
+    """Waiting function, used to wake up the process pool"""
+    setproctitle('oq-worker')
     try:
         import prctl
     except ImportError:
@@ -708,38 +681,41 @@ def wakeup_pool():
         executor.pids = list(pids)
 
 
+# it would be nice to remove this in the future, but it is not easy: the
+# subclasses Sequential and Processmap are used
 class BaseStarmap(object):
     poolfactory = staticmethod(lambda size: multiprocessing.Pool(size))
+    add_task_no = Starmap.__dict__['add_task_no']
+    init = Starmap.__dict__['init']
+    num_tasks = Starmap.__dict__['num_tasks']
 
     @classmethod
-    def apply(cls, func, args, concurrent_tasks=executor._max_workers * 5,
+    def apply(cls, func, args, concurrent_tasks=executor._max_workers * 3,
               weight=lambda item: 1, key=lambda item: 'Unspecified'):
         chunks = split_in_blocks(args[0], concurrent_tasks or 1, weight, key)
         if concurrent_tasks == 0:
             cls = Sequential
         return cls(func, (((chunk,) + args[1:]) for chunk in chunks))
 
-    def __init__(self, func, iterargs, poolsize=None):
+    def __init__(self, func, iterargs, poolsize=None, progress=logging.info):
         self.pool = self.poolfactory(poolsize)
         self.func = func
-        allargs = []
-        for task_no, args in enumerate(iterargs):
-            if isinstance(args[-1], Monitor):
-                # add incremental task number and task weight
-                args[-1].task_no = task_no
-                args[-1].weight = getattr(args[0], 'weight', 1.)
-            allargs.append(args)
-        self.num_tasks = len(allargs)
-        logging.info('Starting %d tasks', self.num_tasks)
+        self.name = func.__name__
+        self.task_args = iterargs
+        self.progress = progress
+        self.init(func)
+        allargs = list(self.add_task_no(iterargs))
+        progress('Starting %s tasks', self.num_tasks)
         self.imap = self.pool.imap_unordered(
             functools.partial(safely_call, func), allargs)
 
-    def submit_all(self, progress=logging.info):
+    def submit_all(self):
         """
         :returns: an :class:`IterResult` instance
         """
         futs = (mkfuture(res) for res in self.imap)
-        return IterResult(futs, self.func.__name__, self.num_tasks, progress)
+        return IterResult(futs, self.func.__name__, self.num_tasks,
+                          self.progress, self.sent)
 
     def __iter__(self):
         try:
@@ -750,10 +726,10 @@ class BaseStarmap(object):
                 self.pool.close()
                 self.pool.join()
 
-    def reduce(self, agg=operator.add, acc=None, progress=logging.info):
+    def reduce(self, agg=operator.add, acc=None):
         if acc is None:
             acc = AccumDict()
-        for res in self.submit_all(progress):
+        for res in self.submit_all():
             acc = agg(acc, res)
         if self.pool:
             self.pool.close()
@@ -765,12 +741,16 @@ class Sequential(BaseStarmap):
     """
     A sequential Starmap, useful for debugging purpose.
     """
-    def __init__(self, func, iterargs, poolsize=None):
+    def __init__(self, func, iterargs, poolsize=None, progress=logging.info):
         self.pool = None
         self.func = func
-        allargs = list(iterargs)
-        self.num_tasks = len(allargs)
-        logging.info('Starting %d sequential tasks', self.num_tasks)
+        self.name = func.__name__
+        self.task_args = iterargs
+        self.progress = progress
+        self.sent = AccumDict()
+        self.argnames = inspect.getargspec(func).args
+        allargs = list(self.add_task_no(iterargs))
+        progress('Starting %s sequential tasks', self.num_tasks)
         self.imap = [safely_call(func, args) for args in allargs]
 
 
@@ -867,7 +847,11 @@ def main(hostport):
     host, port = hostport.split(':')
     conn = Client((host, int(port)))
     func, args = conn.recv()
-    safely_call(func, args, False, conn)
+    res = safely_call(func, args)
+    try:
+        conn.send(res)
+    finally:
+        conn.close()
 
 if __name__ == '__main__':
     main(sys.argv[1])
