@@ -18,8 +18,8 @@
 
 from nose.plugins.attrib import attr
 
-from openquake.qa_tests_data.classical_bcr import case_1
-from openquake.calculators.tests import CalculatorTestCase
+from openquake.qa_tests_data.classical_bcr import case_1, case_2
+from openquake.calculators.tests import CalculatorTestCase, strip_calc_id
 
 
 class ClassicalBCRTestCase(CalculatorTestCase):
@@ -29,3 +29,16 @@ class ClassicalBCRTestCase(CalculatorTestCase):
         out = self.run_calc(case_1.__file__, 'job_risk.ini', exports='csv')
         [fname] = out['bcr-rlzs', 'csv']
         self.assertEqualFiles('expected/bcr-structural.csv', fname)
+
+    @attr('qa', 'risk', 'classical_bcr')
+    def test_case_2(self):
+        # test with the exposure in CSV format
+        self.run_calc(case_2.__file__, 'job.ini',
+                      exposure_file='exposure_model-header.xml')
+
+        # test with the exposure in XML format
+        out = self.run_calc(case_2.__file__, 'job.ini', exports='csv')
+        fnames = out['bcr-stats', 'csv']
+        assert fnames
+        for fname in fnames:
+            self.assertEqualFiles('expected/' + strip_calc_id(fname), fname)
