@@ -18,26 +18,31 @@
 from __future__ import division
 import io
 import ast
+import math
 import os.path
 import numbers
 import operator
 import decimal
 import functools
 import itertools
+import collections
 import numpy
 
 from openquake.baselib.general import (
-    humansize, groupby, AccumDict, CallableDict)
+    humansize, groupby, DictArray, AccumDict, CallableDict)
 from openquake.baselib.performance import perf_dt
+from openquake.baselib.general import get_array
 from openquake.baselib.python3compat import unicode, decode
+from openquake.baselib.general import group_array
 from openquake.hazardlib import valid, stats as hstats
 from openquake.hazardlib.gsim.base import ContextMaker
 from openquake.commonlib import util, source, calc
 from openquake.commonlib.writers import (
     build_header, scientificformat, write_csv, FIVEDIGITS)
+from openquake.calculators import getters
 
 FLOAT = (float, numpy.float32, numpy.float64, decimal.Decimal)
-INT = (int, numpy.uint32, numpy.int64)
+INT = (int, numpy.int32, numpy.uint32, numpy.int64, numpy.uint64)
 F32 = numpy.float32
 U32 = numpy.uint32
 
@@ -218,16 +223,15 @@ def view_contents(token, dstore):
 def view_csm_info(token, dstore):
     csm_info = dstore['csm_info']
     rlzs_assoc = csm_info.get_rlzs_assoc()
-    header = ['smlt_path', 'weight', 'source_model_file',
-              'gsim_logic_tree', 'num_realizations']
+    header = ['smlt_path', 'weight', 'gsim_logic_tree', 'num_realizations']
     rows = []
     for sm in csm_info.source_models:
         num_rlzs = len(rlzs_assoc.rlzs_by_smodel[sm.ordinal])
         num_paths = sm.num_gsim_paths
-        link = "`%s <%s>`_" % (sm.name, sm.name)
-        row = ('_'.join(sm.path), sm.weight, link,
-               classify_gsim_lt(csm_info.gsim_lt), '%d/%d' %
-               (num_rlzs, num_paths))
+        row = ('_'.join(sm.path),
+               sm.weight,
+               classify_gsim_lt(csm_info.gsim_lt),
+               '%d/%d' % (num_rlzs, num_paths))
         rows.append(row)
     return rst_table(rows, header)
 
@@ -235,36 +239,30 @@ def view_csm_info(token, dstore):
 @view.add('ruptures_per_trt')
 def view_ruptures_per_trt(token, dstore):
     tbl = []
-    header = ('source_model grp_id trt num_sources '
-              'eff_ruptures tot_ruptures'.split())
+    header = ('source_model grp_id trt eff_ruptures tot_ruptures'.split())
     num_trts = 0
-    tot_sources = 0
     eff_ruptures = 0
     tot_ruptures = 0
-    source_info = dstore['source_info'].value
     csm_info = dstore['csm_info']
-    r = groupby(source_info, operator.itemgetter('grp_id'),
-                lambda rows: sum(r['num_ruptures'] for r in rows))
-    n = groupby(source_info, operator.itemgetter('grp_id'),
-                lambda rows: sum(1 for r in rows))
+    oq = dstore['oqparam']
+    num_sites = len(dstore['sitecol'])
+    num_tiles = math.ceil(num_sites / oq.sites_per_tile)
     for i, sm in enumerate(csm_info.source_models):
         for src_group in sm.src_groups:
             trt = source.capitalize(src_group.trt)
-            er = src_group.eff_ruptures
+            er = src_group.eff_ruptures / num_tiles
             if er:
                 num_trts += 1
-                num_sources = n.get(src_group.id, 0)
-                tot_sources += num_sources
                 eff_ruptures += er
-                ruptures = r.get(src_group.id, 0)
-                tot_ruptures += ruptures
-                tbl.append((sm.name, src_group.id, trt,
-                            num_sources, er, ruptures))
+                tbl.append(
+                    (sm.names, src_group.id, trt, er, src_group.tot_ruptures))
+            tot_ruptures += src_group.tot_ruptures
     rows = [('#TRT models', num_trts),
-            ('#sources', tot_sources),
             ('#eff_ruptures', eff_ruptures),
             ('#tot_ruptures', tot_ruptures),
-            ('#tot_weight', csm_info.tot_weight), ]
+            ('#tot_weight', csm_info.tot_weight)]
+    if num_tiles > 1:
+        rows.insert(0, ('#tiles', num_tiles))
     if len(tbl) > 1:
         summary = '\n\n' + rst_table(rows)
     else:
@@ -320,10 +318,8 @@ def _humansize(literal):
         items = sorted(dic.items(), key=operator.itemgetter(1), reverse=True)
         lst = ['%s %s' % (k, humansize(v)) for k, v in items]
         return ', '.join(lst)
-    elif isinstance(dic, int):
-        return humansize(dic)
     else:
-        return dic
+        return str(dic)
 
 
 @view.add('job_info')
@@ -345,7 +341,7 @@ def avglosses_data_transfer(token, dstore):
     """
     oq = dstore['oqparam']
     N = len(dstore['assetcol'])
-    R = len(dstore['realizations'])
+    R = dstore['csm_info'].get_num_rlzs()
     L = len(dstore.get_attr('composite_risk_model', 'loss_types'))
     I = oq.insured_losses + 1
     ct = oq.concurrent_tasks
@@ -389,13 +385,14 @@ def view_portfolio_loss(token, dstore):
     """
     oq = dstore['oqparam']
     loss_dt = oq.loss_dt()
-    losses_by_taxon = dstore['losses_by_taxon-rlzs']
-    R = losses_by_taxon.shape[1]  # shape (T, R, L')
+    R = dstore['csm_info'].get_num_rlzs()
+    by_rlzi = group_array(dstore['agg_loss_table'].value, 'rlzi')
     data = numpy.zeros(R, loss_dt)
     rlzids = [str(r) for r in range(R)]
     for r in range(R):
+        loss = by_rlzi[r]['loss'].sum(axis=0)
         for l, lt in enumerate(loss_dt.names):
-            data[r][lt] = losses_by_taxon[:, r, l].sum()
+            data[r][lt] = loss[l]
     array = util.compose_arrays(numpy.array(rlzids), data, 'rlz')
     # this is very sensitive to rounding errors, so I am using a low precision
     return rst_table(array, fmt='%.5E')
@@ -425,7 +422,7 @@ def sum_table(records):
 @view.add('mean_avg_losses')
 def view_mean_avg_losses(token, dstore):
     dt = dstore['oqparam'].loss_dt()
-    weights = dstore['realizations']['weight']
+    weights = dstore['csm_info'].rlzs['weight']
     array = dstore['avg_losses-rlzs'].value  # shape (N, R)
     if len(weights) == 1:  # one realization
         mean = array[:, 0]
@@ -438,33 +435,13 @@ def view_mean_avg_losses(token, dstore):
     return rst_table(losses, fmt=FIVEDIGITS)
 
 
-# this is used by the classical calculator
-@view.add('loss_curves_avg')
-def view_loss_curves_avg(token, dstore):
-    """
-    Returns the average losses computed from the loss curves; for each
-    asset shows all realizations.
-    """
-    array = dstore['loss_curves-rlzs'].value  # shape (N, R)
-    n, r = array.shape
-    lt_dt = numpy.dtype([(lt, numpy.float32, r) for lt in array.dtype.names])
-    avg = numpy.zeros(n, lt_dt)
-    for lt in array.dtype.names:
-        array_lt = array[lt]
-        for i, row in enumerate(array_lt):
-            avg[lt][i] = row['avg']
-    assets = util.get_assets(dstore)
-    losses = util.compose_arrays(assets, avg)
-    return rst_table(losses, fmt='%8.6E')
-
-
 @view.add('exposure_info')
 def view_exposure_info(token, dstore):
     """
     Display info about the exposure model
     """
     assetcol = dstore['assetcol/array'][:]
-    taxonomies = dstore['assetcol/taxonomies'][:]
+    taxonomies = sorted(set(dstore['assetcol'].taxonomies))
     cc = dstore['assetcol/cost_calculator']
     ra_flag = ['relative', 'absolute']
     data = [('#assets', len(assetcol)),
@@ -475,27 +452,10 @@ def view_exposure_info(token, dstore):
     return rst_table(data) + '\n\n' + view_assets_by_site(token, dstore)
 
 
-@view.add('assetcol')
-def view_assetcol(token, dstore):
-    """
-    Display the exposure in CSV format
-    """
-    assetcol = dstore['assetcol'].value
-    taxonomies = dstore['assetcol/taxonomies'].value
-    header = list(assetcol.dtype.names)
-    columns = [None] * len(header)
-    for i, field in enumerate(header):
-        if field == 'taxonomy':
-            columns[i] = taxonomies[assetcol[field]]
-        else:
-            columns[i] = assetcol[field]
-    return write_csv(io.StringIO(), [header] + list(zip(*columns)))
-
-
 @view.add('ruptures_events')
 def view_ruptures_events(token, dstore):
-    num_ruptures = sum(len(v) for v in dstore['ruptures'].values())
-    num_events = sum(len(v) for v in dstore['events'].values())
+    num_ruptures = len(dstore['ruptures'])
+    num_events = len(dstore['events'])
     mult = round(num_events / num_ruptures, 3)
     lst = [('Total number of ruptures', num_ruptures),
            ('Total number of events', num_events),
@@ -547,15 +507,31 @@ def stats(name, array, *extras):
     :param name: a descriptive string
     :returns: (name, mean, std, min, max, len)
     """
-    return (name, numpy.mean(array), numpy.std(array, ddof=1),
+    std = numpy.nan if len(array) == 1 else numpy.std(array, ddof=1)
+    return (name, numpy.mean(array), std,
             numpy.min(array), numpy.max(array), len(array)) + extras
 
 
+@view.add('num_units')
+def view_num_units(token, dstore):
+    """
+    Display the number of units by taxonomy
+    """
+    taxo = dstore['assetcol/tagcol/taxonomy'].value
+    counts = collections.Counter()
+    for asset in dstore['assetcol']:
+        counts[taxo[asset.taxonomy]] += asset.number
+    data = sorted(counts.items())
+    data.append(('*ALL*', sum(d[1] for d in data)))
+    return rst_table(data, header=['taxonomy', 'num_units'])
+
+
 @view.add('assets_by_site')
 def view_assets_by_site(token, dstore):
     """
     Display statistical information about the distribution of the assets
     """
+    taxonomies = dstore['assetcol/tagcol/taxonomy'].value
     assets_by_site = dstore['assetcol'].assets_by_site()
     data = ['taxonomy mean stddev min max num_sites num_assets'.split()]
     num_assets = AccumDict()
@@ -564,7 +540,7 @@ def view_assets_by_site(token, dstore):
             assets, operator.attrgetter('taxonomy')).items()}
     for taxo in sorted(num_assets):
         val = numpy.array(num_assets[taxo])
-        data.append(stats(taxo, val, val.sum()))
+        data.append(stats(taxonomies[taxo], val, val.sum()))
     if len(num_assets) > 1:  # more than one taxonomy, add a summary
         n_assets = numpy.array([len(assets) for assets in assets_by_site])
         data.append(stats('*ALL*', n_assets, n_assets.sum()))
@@ -576,10 +552,10 @@ def view_required_params_per_trt(token, dstore):
     """
     Display the parameters needed by each tectonic region type
     """
-    gsims_per_grp_id = sorted(
-        dstore['csm_info'].get_rlzs_assoc().gsims_by_grp_id.items())
+    csm_info = dstore['csm_info']
     tbl = []
-    for grp_id, gsims in gsims_per_grp_id:
+    for grp_id, trt in sorted(csm_info.grp_by("trt").items()):
+        gsims = csm_info.gsim_lt.get_gsims(trt)
         maker = ContextMaker(gsims)
         distances = sorted(maker.REQUIRES_DISTANCES)
         siteparams = sorted(maker.REQUIRES_SITES_PARAMETERS)
@@ -610,8 +586,9 @@ def view_task_info(token, dstore):
 
     data = ['operation-duration mean stddev min max num_tasks'.split()]
     for task in dstore['task_info']:
-        val = dstore['task_info/' + task]['duration']
-        data.append(stats(task, val))
+        if task not in ('task_sources', 'source_data'):  # this is special
+            val = dstore['task_info/' + task]['duration']
+            data.append(stats(task, val))
     if len(data) == 1:
         return 'Not available'
     return rst_table(data)
@@ -629,17 +606,25 @@ def view_task_durations(token, dstore):
     return '\n'.join(map(str, array))
 
 
-@view.add('task_slowest')
-def view_task_slowest(token, dstore):
+@view.add('task')
+def view_task(token, dstore):
     """
-    Display info about the slowest classical task.
+    Display info about a given task. Here are a few examples of usage::
+
+     $ oq show task:0  # the fastest task
+     $ oq show task:-1  # the slowest task
     """
-    i = dstore['task_info/classical']['duration'].argmax()
-    taskno, weight, duration = dstore['task_info/classical'][i]
-    sources = dstore['task_sources'][taskno - 1].split()
+    data = dstore['task_info/classical'].value
+    data.sort(order='duration')
+    i = int(token.split(':')[1])
+    taskno, weight, duration = data[i]
+    arr = get_array(dstore['task_info/source_data'].value, taskno=taskno)
+    st = [stats('nsites', arr['nsites']), stats('weight', arr['weight'])]
+    sources = dstore['task_info/task_sources'][taskno - 1].split()
     srcs = set(decode(s).split(':', 1)[0] for s in sources)
-    return 'taskno=%d, weight=%d, duration=%d s, sources="%s"' % (
+    res = 'taskno=%d, weight=%d, duration=%d s, sources="%s"\n\n' % (
         taskno, weight, duration, ' '.join(sorted(srcs)))
+    return res + rst_table(st, header='variable mean stddev min max n'.split())
 
 
 @view.add('hmap')
@@ -666,27 +651,129 @@ def view_hmap(token, dstore):
     return rst_table(array)
 
 
-@view.add('synthetic_hcurves')
-def view_synthetic_hcurves(token, dstore):
+@view.add('flat_hcurves')
+def view_flat_hcurves(token, dstore):
     """
-    Display the synthetic hazard curves for the calculation. They are
+    Display the flat hazard curves for the calculation. They are
     used for debugging purposes when comparing the results of two
-    calculations, they have no physical meaning. They are the simple mean
-    of the PoEs arrays over source groups, gsims and number of sites.
+    calculations. They are the mean over the sites of the mean hazard
+    curves.
     """
     oq = dstore['oqparam']
     nsites = len(dstore['sitecol'])
-    array = numpy.zeros(len(oq.imtls.array), F32)
-    ngroups = 0
-    for sm in dstore['csm_info'].source_models:
-        for src_group in sm.src_groups:
-            grp_id = src_group.id
-            try:
-                pmap = dstore['poes/grp-%02d' % grp_id]
-            except KeyError:
-                continue
-            ngroups += 1
-            for sid in pmap:
-                array += pmap[sid].array.sum(axis=1) / pmap.shape_z
-    array /= (ngroups * nsites)
-    return oq.imtls.new(array)
+    mean = getters.PmapGetter(dstore).get_mean()
+    array = calc.convert_to_array(mean, nsites, oq.imtls)
+    res = numpy.zeros(1, array.dtype)
+    for name in array.dtype.names:
+        res[name] = array[name].mean()
+    return rst_table(res)
+
+
+@view.add('flat_hmaps')
+def view_flat_hmaps(token, dstore):
+    """
+    Display the flat hazard maps for the calculation. They are
+    used for debugging purposes when comparing the results of two
+    calculations. They are the mean over the sites of the mean hazard
+    maps.
+    """
+    oq = dstore['oqparam']
+    assert oq.poes
+    nsites = len(dstore['sitecol'])
+    pdic = DictArray({imt: oq.poes for imt in oq.imtls})
+    mean = getters.PmapGetter(dstore).get_mean()
+    hmaps = calc.make_hmap(mean, oq.imtls, oq.poes)
+    array = calc.convert_to_array(hmaps, nsites, pdic)
+    res = numpy.zeros(1, array.dtype)
+    for name in array.dtype.names:
+        res[name] = array[name].mean()
+    return rst_table(res)
+
+
+@view.add('dupl_sources')
+def view_dupl_sources(token, dstore):
+    """
+    Display the duplicated sources from source_info
+    """
+    info = dstore['source_info']
+    items = sorted(group_array(info.value, 'source_id').items())
+    tbl = []
+    tot_calc_time = 0
+    for source_id, records in items:
+        if len(records) > 1:  # dupl
+            calc_time = records['calc_time'].sum()
+            tot_calc_time += calc_time
+            grp_ids = sorted(rec['grp_id'] for rec in records)
+            tbl.append((source_id, calc_time, grp_ids))
+    if tbl and info.attrs['has_dupl_sources']:
+        tot = info['calc_time'].sum()
+        percent = tot_calc_time / tot * 100
+        m = '\nTotal calc_time in duplicated sources: %d/%d (%d%%)' % (
+            tot_calc_time, tot, percent)
+        return rst_table(tbl, ['source_id', 'calc_time', 'src_group_ids']) + m
+    else:
+        return 'There are no duplicated sources'
+
+
+@view.add('global_poes')
+def view_global_poes(token, dstore):
+    """
+    Display global probabilities averaged on all sites and all GMPEs
+    """
+    tbl = []
+    imtls = dstore['oqparam'].imtls
+    header = ['grp_id'] + [str(poe) for poe in imtls.array]
+    for grp in sorted(dstore['poes']):
+        poes = dstore['poes/' + grp]
+        nsites = len(poes)
+        site_avg = sum(poes[sid].array for sid in poes) / nsites
+        gsim_avg = site_avg.sum(axis=1) / poes.shape_z
+        tbl.append([grp] + list(gsim_avg))
+    return rst_table(tbl, header=header)
+
+
+@view.add('mean_disagg')
+def view_mean_disagg(token, dstore):
+    """
+    Display mean quantities for the disaggregation. Useful for checking
+    differences between two calculations.
+    """
+    tbl = []
+    for key, dset in sorted(dstore['disagg'].items()):
+        vals = [ds.value.mean() for k, ds in sorted(dset.items())]
+        tbl.append([key] + vals)
+    header = ['key'] + sorted(dset)
+    return rst_table(sorted(tbl), header=header)
+
+
+@view.add('elt')
+def view_elt(token, dstore):
+    """
+    Display the event loss table averaged by event
+    """
+    oq = dstore['oqparam']
+    R = len(dstore['csm_info'].rlzs)
+    dic = group_array(dstore['agg_loss_table'].value, 'rlzi')
+    header = oq.loss_dt().names
+    tbl = []
+    for rlzi in range(R):
+        if rlzi in dic:
+            tbl.append(dic[rlzi]['loss'].mean(axis=0))
+        else:
+            tbl.append([0.] * len(header))
+    return rst_table(tbl, header)
+
+
+@view.add('pmap')
+def view_pmap(token, dstore):
+    """
+    Display the mean ProbabilityMap associated to a given source group name
+    """
+    name = token.split(':')[1]  # called as pmap:name
+    pmap = {}
+    pgetter = getters.PmapGetter(dstore)
+    for grp, dset in dstore['poes'].items():
+        if dset.attrs['name'] == name:
+            pmap = pgetter.get_mean(grp)
+            break
+    return str(pmap)
