@@ -28,13 +28,12 @@ import os
 import re
 import sys
 import copy
-import random
 import itertools
 import collections
 import operator
 from collections import namedtuple
 from decimal import Decimal
-
+import numpy
 from openquake.baselib.general import groupby
 from openquake.baselib.python3compat import raise_
 import openquake.hazardlib.source as ohs
@@ -59,9 +58,9 @@ class SourceModel(object):
     A container of SourceGroup instances with some additional attributes
     describing the source model in the logic tree.
     """
-    def __init__(self, name, weight, path, src_groups, num_gsim_paths, ordinal,
-                 samples):
-        self.name = name
+    def __init__(self, names, weight, path, src_groups, num_gsim_paths,
+                 ordinal, samples):
+        self.names = names
         self.weight = weight
         self.path = path
         self.src_groups = src_groups
@@ -69,8 +68,24 @@ class SourceModel(object):
         self.ordinal = ordinal
         self.samples = samples
 
+    @property
+    def name(self):
+        """
+        Compact representation for the names
+        """
+        names = self.names.split()
+        if len(names) == 1:
+            return names[0]
+        elif len(names) == 2:
+            return ' '.join(names)
+        else:
+            return ' '.join([names[0], '...', names[-1]])
+
     @property
     def num_sources(self):
+        """
+        Number of sources contained in the source model
+        """
         return sum(len(sg) for sg in self.src_groups)
 
     def get_skeleton(self):
@@ -83,13 +98,13 @@ class SourceModel(object):
             sg = copy.copy(grp)
             sg.sources = []
             src_groups.append(sg)
-        return self.__class__(self.name, self.weight, self.path, src_groups,
+        return self.__class__(self.names, self.weight, self.path, src_groups,
                               self.num_gsim_paths, self.ordinal, self.samples)
 
     def __repr__(self):
         samples = ', samples=%d' % self.samples if self.samples > 1 else ''
         return '<%s #%d %s, path=%s, weight=%s%s>' % (
-            self.__class__.__name__, self.ordinal, self.name,
+            self.__class__.__name__, self.ordinal, self.names,
             '_'.join(self.path), self.weight, samples)
 
 Realization = namedtuple('Realization', 'value weight lt_path ordinal lt_uid')
@@ -153,21 +168,7 @@ class ValidationError(LogicTreeError):
             self.filename, self.lineno, self.message)
 
 
-# private function used in sample
-def sample_one(branches, rnd):
-    # Draw a random number and iterate through the branches in the set
-    # (adding up their weights) until the random value falls into
-    # the interval occupied by a branch. Return the latter.
-    diceroll = rnd.random()
-    acc = 0
-    for branch in branches:
-        acc += branch.weight
-        if acc >= diceroll:
-            return branch
-    raise AssertionError('do weights really sum up to 1.0?')
-
-
-def sample(weighted_objects, num_samples, rnd):
+def sample(weighted_objects, num_samples, seed):
     """
     Take random samples of a sequence of weighted objects
 
@@ -176,16 +177,16 @@ def sample(weighted_objects, num_samples, rnd):
         The weights must sum up to 1.
     :param num_samples:
         The number of samples to return
-    :param rnd:
-        Random object. Should have method ``random()`` -- return uniformly
-        distributed random float number >= 0 and < 1.
+    :param seed:
+        A random seed
     :return:
         A subsequence of the original sequence with `num_samples` elements
     """
-    subsequence = []
-    for _ in range(num_samples):
-        subsequence.append(sample_one(weighted_objects, rnd))
-    return subsequence
+    weights = numpy.array([float(obj.weight) for obj in weighted_objects])
+    numpy.random.seed(seed)
+    idxs = numpy.random.choice(len(weights), num_samples, p=weights)
+    # NB: returning an array would break things
+    return [weighted_objects[idx] for idx in idxs]
 
 
 class Branch(object):
@@ -444,6 +445,46 @@ class BranchSet(object):
                                        occurrence_rates=occur_rates))
 
 
+class FakeSmlt(object):
+    """
+    A replacement for the SourceModelLogicTree class, to be used when
+    there is a trivial source model logic tree. In practice, when
+    `source_model_logic_tree_file` is missing but there is a
+    `source_model_file` in the job.ini file.
+    """
+    def __init__(self, filename, seed=0, num_samples=0):
+        self.filename = filename
+        self.basepath = os.path.dirname(filename)
+        self.seed = seed
+        self.num_samples = num_samples
+        self.tectonic_region_types = set()
+
+    def gen_source_models(self, gsim_lt):
+        """
+        Yield the underlying SourceModel, multiple times if there is sampling
+        """
+        num_gsim_paths = 1 if self.num_samples else gsim_lt.get_num_paths()
+        for i, rlz in enumerate(self):
+            yield SourceModel(
+                rlz.value, rlz.weight, ('b1',), [], num_gsim_paths, i, 1)
+
+    def make_apply_uncertainties(self, branch_ids):
+        """
+        :returns: a do nothing function
+        """
+        return lambda source: None
+
+    def __iter__(self):
+        name = os.path.basename(self.filename)
+        smlt_path = ('b1',)
+        if self.num_samples:  # many realizations of equal weight
+            weight = 1. / self.num_samples
+            for i in range(self.num_samples):
+                yield Realization(name, weight, smlt_path, None, smlt_path)
+        else:  # there is a single realization
+            yield Realization(name, 1.0, smlt_path, 0, smlt_path)
+
+
 class SourceModelLogicTree(object):
     """
     Source model logic tree parser.
@@ -613,16 +654,16 @@ class SourceModelLogicTree(object):
                 rlz.value, rlz.weight / num_samples, smpath, [],
                 num_gsim_paths, i, num_samples)
 
-    def sample_path(self, rnd):
+    def sample_path(self, seed):
         """
         Return the model name and a list of branch ids.
 
-        :param int random_seed: the seed used for the sampling
+        :param seed: the seed used for the sampling
         """
         branchset = self.root_branchset
         branch_ids = []
         while branchset is not None:
-            [branch] = sample(branchset.branches, 1, rnd)
+            [branch] = sample(branchset.branches, 1, seed)
             branch_ids.append(branch.branch_id)
             branchset = branch.child_branchset
         modelname = self.root_branchset.get_branch_by_id(branch_ids[0]).value
@@ -630,25 +671,25 @@ class SourceModelLogicTree(object):
 
     def __iter__(self):
         """
-        Yield Realization tuples. Notice that
-        weight is not None only when the number_of_logic_tree_samples
-        is 0. In that case a full enumeration is performed, otherwise
-        a random sampling is performed.
+        Yield Realization tuples. Notice that the weight is homogeneous when
+        sampling is enabled, since it is accounted for in the sampling
+        procedure.
         """
         if self.num_samples:
             # random sampling of the logic tree
-            rnd = random.Random(self.seed)
             weight = 1. / self.num_samples
-            for _ in range(self.num_samples):
-                name, sm_lt_path = self.sample_path(rnd)
+            for i in range(self.num_samples):
+                name, sm_lt_path = self.sample_path(self.seed + i)
                 yield Realization(name, weight, tuple(sm_lt_path), None,
                                   tuple(sm_lt_path))
         else:  # full enumeration
+            ordinal = 0
             for weight, smlt_path in self.root_branchset.enumerate_paths():
                 name = smlt_path[0].value
                 smlt_branch_ids = [branch.branch_id for branch in smlt_path]
-                yield Realization(name, weight, tuple(smlt_branch_ids), None,
-                                  tuple(smlt_branch_ids))
+                yield Realization(name, weight, tuple(smlt_branch_ids),
+                                  ordinal, tuple(smlt_branch_ids))
+                ordinal += 1
 
     def parse_uncertainty_value(self, node, branchset):
         """
@@ -1179,7 +1220,7 @@ class GsimLogicTree(object):
         """
         :returns: an XML string representing the logic tree
         """
-        return nrml.convert(self._ltnode)
+        return nrml.to_string(self._ltnode)
 
     def reduce(self, trts):
         """
@@ -1250,16 +1291,21 @@ class GsimLogicTree(object):
                     weights.append(weight)
                     branch_id = branch['branchID']
                     uncertainty = branch.uncertaintyModel
-                    gsim_name = uncertainty.text.strip()
-                    if gsim_name == 'GMPETable':
-                        # a bit hackish: set the GMPE_DIR equal to the
-                        # directory where the gsim_logic_tree file is
-                        GMPETable.GMPE_DIR = os.path.dirname(self.fname)
-                    try:
-                        gsim = valid.gsim(gsim_name, **uncertainty.attrib)
-                    except:
-                        etype, exc, tb = sys.exc_info()
-                        raise_(etype, "%s in file %s" % (exc, self.fname), tb)
+                    if hasattr(uncertainty.text, 'strip'):  # a string
+                        gsim_name = uncertainty.text.strip()
+                        if gsim_name == 'GMPETable':
+                            # a bit hackish: set the GMPE_DIR equal to the
+                            # directory where the gsim_logic_tree file is
+                            GMPETable.GMPE_DIR = os.path.dirname(self.fname)
+                        try:
+                            gsim = valid.gsim(gsim_name, **uncertainty.attrib)
+                        except:
+                            etype, exc, tb = sys.exc_info()
+                            raise_(etype, "%s in file %s" % (exc, self.fname),
+                                   tb)
+                        uncertainty.text = gsim
+                    else:  # already converted GSIM
+                        gsim = uncertainty.text
                     self.values[trt].append(gsim)
                     bt = BranchTuple(
                         branchset, branch_id, gsim, weight, effective)
@@ -1269,6 +1315,7 @@ class GsimLogicTree(object):
             raise InvalidLogicTree(
                 'Found duplicated applyToTectonicRegionType=%s' % trts)
         branches.sort(key=lambda b: (b.bset['branchSetID'], b.id))
+        # TODO: add an .idx to each GSIM ?
         return trts, branches
 
     def get_gsim_by_trt(self, rlz, trt):
@@ -1282,6 +1329,20 @@ class GsimLogicTree(object):
         idx = self.all_trts.index(trt)
         return rlz.value[idx]
 
+    def get_gsims(self, trt, rlzs=None):
+        """
+        :param trt: tectonic region type
+        :param rlzs: a sequence of realization indices (or None)
+        :returns: sorted list of available GSIMs for that trt
+        """
+        if rlzs is None:
+            if trt == '*' or trt == b'*':  # fake logictree
+                [trt] = self.values
+            gsims = self.values[trt]
+        else:
+            gsims = set(self.get_gsim_by_trt(rlz, trt) for rlz in rlzs)
+        return sorted(gsims)
+
     def __iter__(self):
         """
         Yield :class:`openquake.commonlib.logictree.Realization` instances
