@@ -21,6 +21,7 @@ Module :mod:`openquake.hazardlib.geo.utils` contains functions that are common
 to several geographical primitives and some other low-level spatial operations.
 """
 import operator
+import collections
 try:
     import rtree
 except:
@@ -33,6 +34,8 @@ from openquake.hazardlib.geo.geodetic import (
     EARTH_RADIUS, geodetic_distance, min_idx_dst)
 from openquake.baselib.slots import with_slots
 
+SphericalBB = collections.namedtuple('SphericalBB', 'west east north south')
+
 
 class GeographicObjects(object):
     """
@@ -58,11 +61,10 @@ class GeographicObjects(object):
             for i, (x, y) in enumerate(zip(xs, ys)):
                 self.index.insert(i, (x, y, x, y))
 
-    def get_closest(self, lon, lat, max_distance=None):
+    def get_closest(self, lon, lat):
         """
         Get the closest object to the given longitude and latitude
-        and its distance. If the `max_distance` is given and all objects
-        are farther than the maximum distance, returns (None, None).
+        and its distance.
 
         :param lon: longitude in degrees
         :param lat: latitude in degrees
@@ -76,8 +78,6 @@ class GeographicObjects(object):
         else:
             zeros = numpy.zeros_like(self.lons)
             idx, min_dist = min_idx_dst(self.lons, self.lats, zeros, lon, lat)
-        if max_distance is not None and min_dist > max_distance:
-            return None, None
         return self.objects[idx], min_dist
 
 
@@ -180,7 +180,7 @@ def get_spherical_bounding_box(lons, lats):
                    for lon in lons):
             raise ValueError('points collection has longitudinal extent '
                              'wider than 180 deg')
-    return west, east, north, south
+    return SphericalBB(west, east, north, south)
 
 
 @with_slots
@@ -480,16 +480,3 @@ def plane_fit(points):
     M = numpy.dot(x, x.T)
     return ctr, numpy.linalg.svd(M)[0][:, -1]
 
-
-def fix_lons_idl(lons):
-    """
-    Fix a vector of longitudes crossing the International Date Line (if any).
-
-    :returns: the fixed vector and an IDL flag
-    """
-    if cross_idl(lons.min(), lons.max()):
-        new = numpy.array(lons)
-        new[new < 0] += 360
-        return new, True
-    else:
-        return lons, False
