@@ -393,10 +393,11 @@ class GMPETable(GMPE):
             self.REQUIRES_SITES_PARAMETERS = set(
                 [self.amplification.parameter])
         elif self.amplification.element == "Rupture":
-            # Re-set the site parameters
+            # set the site and rupture parameters on the instance
             self.REQUIRES_SITES_PARAMETERS = set()
-            self.REQUIRES_RUPTURE_PARAMETERS.add(
-                self.amplification.parameter)
+            self.REQUIRES_RUPTURE_PARAMETERS = (
+                self.REQUIRES_RUPTURE_PARAMETERS | set(
+                    [self.amplification.parameter]))
 
     def _supported_imts(self):
         """
@@ -524,8 +525,8 @@ class GMPETable(GMPE):
 
             low_period = round(periods[0], 7)
             high_period = round(periods[-1], 7)
-
-            if imt.period < low_period or imt.period > high_period:
+            if (round(imt.period, 7) < low_period) or\
+                (round(imt.period, 7) > high_period):
                 raise ValueError("Spectral period %.3f outside of valid range "
                                  "(%.3f to %.3f)" % (imt.period, periods[0],
                                                      periods[-1]))
