@@ -23,7 +23,6 @@ Validation library for the engine, the desktop tools, and anything else
 import re
 import ast
 import logging
-import textwrap
 import collections
 from decimal import Decimal
 import numpy
@@ -32,6 +31,7 @@ from openquake.baselib.python3compat import with_metaclass
 from openquake.baselib.general import distinct
 from openquake.baselib import hdf5
 from openquake.hazardlib import imt, scalerel, gsim
+from openquake.hazardlib.gsim.gsim_table import GMPETable
 from openquake.hazardlib.calc import disagg
 from openquake.hazardlib.calc.filters import IntegrationDistance
 
@@ -58,6 +58,11 @@ def disagg_outputs(value):
     return values
 
 
+class FromFile(object):  # fake GSIM
+    def __str__(self):
+        return 'FromFile'
+
+
 # more tests are in tests/valid_test.py
 def gsim(value, **kwargs):
     """
@@ -66,18 +71,32 @@ def gsim(value, **kwargs):
     >>> gsim('BooreAtkinson2011')
     'BooreAtkinson2011()'
     """
-    if value == 'FromFile':
-        return 'FromFile'
-    elif value.endswith('()'):
+    minimum_distance = float(kwargs.pop('minimum_distance', 0))
+    if value.endswith('()'):
         value = value[:-2]  # strip parenthesis
+    if value == 'FromFile':
+        return FromFile()
+    elif value.startswith('GMPETable'):
+        gsim_class = GMPETable
+    else:
+        try:
+            gsim_class = GSIM[value]
+        except KeyError:
+            raise ValueError('Unknown GSIM: %s' % value)
     try:
-        gsim_class = GSIM[value]
-    except KeyError:
-        raise ValueError('Unknown GSIM: %s' % value)
-    try:
-        return gsim_class(**kwargs)
+        gs = gsim_class(**kwargs)
     except TypeError:
         raise ValueError('Could not instantiate %s%s' % (value, kwargs))
+    gs.minimum_distance = minimum_distance
+    return gs
+
+
+def logic_tree_path(value):
+    """
+    >>> logic_tree_path('SM2_a3b1')
+    ['SM2', 'a3b1']
+    """
+    return value.split('_')
 
 
 def compose(*validators):
@@ -240,24 +259,29 @@ ASSET_ID_LENGTH = 100
 simple_id = SimpleId(MAX_ID_LENGTH)
 asset_id = SimpleId(ASSET_ID_LENGTH)
 source_id = SimpleId(MAX_ID_LENGTH, r'^[\w\.\-_]+$')
+nice_string = SimpleId(  # nice for Windows, Linux, HDF5 and XML
+    ASSET_ID_LENGTH, r'[a-zA-Z0-9\.`!#$%\(\)\+/,;@\[\]\^_{|}~-]+')
 
 
 class FloatRange(object):
-    def __init__(self, minrange, maxrange):
+    def __init__(self, minrange, maxrange, name=''):
         self.minrange = minrange
         self.maxrange = maxrange
+        self.name = name
         self.__name__ = 'FloatRange[%s:%s]' % (minrange, maxrange)
 
     def __call__(self, value):
         f = float_(value)
         if f > self.maxrange:
-            raise ValueError("'%s' is bigger than the max, '%s'" %
-                             (f, self.maxrange))
+            raise ValueError("%s %s is bigger than the maximum (%s)" %
+                             (self.name, f, self.maxrange))
         if f < self.minrange:
-            raise ValueError("'%s' is smaller than the min, '%s'" %
-                             (f, self.minrange))
+            raise ValueError("%s %s is smaller than the minimum (%s)" %
+                             (self.name, f, self.minrange))
         return f
 
+magnitude = FloatRange(0, 11, 'magnitude')
+
 
 def not_empty(value):
     """Check that the string is not all blanks"""
@@ -634,10 +658,10 @@ def check_levels(imls, imt):
     :param imt: the intensity measure type
 
     >>> check_levels([0.1, 0.2], 'PGA')  # ok
-    >>> check_levels([0.1], 'PGA')
+    >>> check_levels([], 'PGA')
     Traceback (most recent call last):
        ...
-    ValueError: Not enough imls for PGA: [0.1]
+    ValueError: No imls for PGA: []
     >>> check_levels([0.2, 0.1], 'PGA')
     Traceback (most recent call last):
        ...
@@ -647,8 +671,8 @@ def check_levels(imls, imt):
        ...
     ValueError: Found duplicated levels for PGA: [0.2, 0.2]
     """
-    if len(imls) < 2:
-        raise ValueError('Not enough imls for %s: %s' % (imt, imls))
+    if len(imls) < 1:
+        raise ValueError('No imls for %s: %s' % (imt, imls))
     elif imls != sorted(imls):
         raise ValueError('The imls for %s are not sorted: %s' % (imt, imls))
     elif len(distinct(imls)) < len(imls):
@@ -774,7 +798,13 @@ def maximum_distance(value):
     :returns:
         a IntegrationDistance mapping
     """
-    return IntegrationDistance(floatdict(value))
+    dic = floatdict(value)
+    for trt, magdists in dic.items():
+        if isinstance(magdists, list):  # could be a scalar otherwise
+            magdists.sort()  # make sure the list is sorted by magnitude
+            for mag, dist in magdists:  # validate the magnitudes
+                magnitude(mag)
+    return IntegrationDistance(dic)
 
 
 # ########################### SOURCES/RUPTURES ############################# #
@@ -923,6 +953,8 @@ def integers(value):
        ...
     ValueError: Not a list of integers: ' '
     """
+    if '.' in value:
+        raise ValueError('There are decimal points in %s' % value)
     values = value.replace(',', ' ').split()
     if not values:
         raise ValueError('Not a list of integers: %r' % value)
@@ -1148,8 +1180,9 @@ class ParamSet(with_metaclass(MetaParamSet, hdf5.LiteralAttrs)):
                   if valid.startswith('is_valid_')]
         for is_valid in valids:
             if not is_valid():
-                docstring = is_valid.__doc__.strip()
-                doc = textwrap.fill(docstring.format(**vars(self)))
+                docstring = '\n'.join(
+                    line.strip() for line in is_valid.__doc__.splitlines())
+                doc = docstring.format(**vars(self))
                 raise ValueError(doc)
 
     def __iter__(self):
