@@ -20,14 +20,26 @@ import shutil
 import json
 import logging
 import os
-import getpass
+import sys
+import inspect
 import tempfile
+import subprocess
+import threading
+import signal
+import zlib
 try:
     import urllib.parse as urlparse
 except ImportError:
     import urlparse
 import re
-
+import numpy
+import psutil
+try:
+    # Python 3
+    from urllib.parse import unquote_plus
+except ImportError:
+    # Python 2
+    from urllib import unquote_plus
 from xml.parsers.expat import ExpatError
 from django.http import (
     HttpResponse, HttpResponseNotFound, HttpResponseBadRequest)
@@ -35,19 +47,21 @@ from django.views.decorators.csrf import csrf_exempt
 from django.views.decorators.http import require_http_methods
 from django.shortcuts import render
 
+from openquake.baselib import datastore
 from openquake.baselib.general import groupby, writetmp
-from openquake.baselib.python3compat import unicode
-from openquake.baselib.parallel import Starmap, safely_call
+from openquake.baselib.python3compat import unicode, pickle
+from openquake.baselib.parallel import safely_call
 from openquake.hazardlib import nrml, gsim
 from openquake.risklib import read_nrml
 
-from openquake.commonlib import readinput, oqvalidation, logs, datastore
+from openquake.commonlib import readinput, oqvalidation, logs
 from openquake.calculators.export import export
+from openquake.calculators.extract import extract as _extract
 from openquake.engine import __version__ as oqversion
 from openquake.engine.export import core
 from openquake.engine import engine
 from openquake.engine.export.core import DataStoreExportError
-from openquake.server import executor, utils, dbapi
+from openquake.server import utils, dbapi
 
 from django.conf import settings
 if settings.LOCKDOWN:
@@ -61,6 +75,7 @@ try:
 except ImportError:
     from django.core.servers.basehttp import FileWrapper
 
+
 read_nrml.update_validators()  # update risk validators
 
 METHOD_NOT_ALLOWED = 405
@@ -192,6 +207,16 @@ def get_engine_version(request):
     return HttpResponse(oqversion)
 
 
+@cross_domain_ajax
+@require_http_methods(['GET'])
+def get_engine_latest_version(request):
+    """
+    Return a string with if new versions have been released.
+    Return 'None' if the version is not available
+    """
+    return HttpResponse(engine.check_obsolete_version())
+
+
 @cross_domain_ajax
 @require_http_methods(['GET'])
 def get_available_gsims(request):
@@ -235,7 +260,7 @@ def validate_nrml(request):
             'Please provide the "xml_text" parameter')
     xml_file = writetmp(xml_text, suffix='.xml')
     try:
-        nrml.parse(xml_file)
+        nrml.to_python(xml_file)
     except ExpatError as exc:
         return _make_response(error_msg=str(exc),
                               error_line=exc.lineno,
@@ -283,9 +308,10 @@ def calc_info(request, calc_id):
 
 @require_http_methods(['GET'])
 @cross_domain_ajax
-def calc(request, id=None):
+def calc_list(request, id=None):
+    # view associated to the endpoints /v1/calc/list and /v1/calc/:id/status
     """
-    Get a list of calculations and report their id, status, job_type,
+    Get a list of calculations and report their id, status, calculation_mode,
     is_running, description, and a url where more detailed information
     can be accessed. This is called several times by the Javascript.
 
@@ -294,16 +320,27 @@ def calc(request, id=None):
     base_url = _get_base_url(request)
 
     user = utils.get_user_data(request)
-
+    allowed_users = user['group_members'] or [user['name']]
     calc_data = logs.dbcmd('get_calcs', request.GET,
-                           user['name'], user['acl_on'], id)
+                           allowed_users, user['acl_on'], id)
 
     response_data = []
-    for hc_id, owner, status, job_type, is_running, desc in calc_data:
+    for hc_id, owner, status, calculation_mode, is_running, desc, pid \
+            in calc_data:
         url = urlparse.urljoin(base_url, 'v1/calc/%d' % hc_id)
+        abortable = False
+        if is_running:
+            try:
+                if (psutil.Process(pid).username() ==
+                        psutil.Process(os.getpid()).username()):
+                    abortable = True
+            except psutil.NoSuchProcess:
+                pass
         response_data.append(
-            dict(id=hc_id, owner=owner, status=status, job_type=job_type,
-                 is_running=bool(is_running), description=desc, url=url))
+            dict(id=hc_id, owner=owner,
+                 calculation_mode=calculation_mode, status=status,
+                 is_running=bool(is_running), description=desc, url=url,
+                 abortable=abortable))
 
     # if id is specified the related dictionary is returned instead the list
     if id is not None:
@@ -313,6 +350,46 @@ def calc(request, id=None):
                         content_type=JSON)
 
 
+@csrf_exempt
+@cross_domain_ajax
+@require_http_methods(['POST'])
+def calc_abort(request, calc_id):
+    """
+    Abort the given calculation, it is it running
+    """
+    job = logs.dbcmd('get_job', calc_id)
+    if job is None:
+        message = {'error': 'Unknown job %s' % calc_id}
+        return HttpResponse(content=json.dumps(message), content_type=JSON)
+
+    if job.status not in ('executing', 'running'):
+        message = {'error': 'Job %s is not running' % job.id}
+        return HttpResponse(content=json.dumps(message), content_type=JSON)
+
+    user = utils.get_user_data(request)
+    info = logs.dbcmd('calc_info', calc_id)
+    allowed_users = user['group_members'] or [user['name']]
+    if user['acl_on'] and info['user_name'] not in allowed_users:
+        message = {'error': ('User %s has no permission to abort job %s' %
+                             (info['user_name'], job.id))}
+        return HttpResponse(content=json.dumps(message), content_type=JSON,
+                            status=403)
+
+    if job.pid:  # is a spawned job
+        try:
+            os.kill(job.pid, signal.SIGTERM)
+        except Exception as exc:
+            logging.error(exc)
+        else:
+            logging.warn('Aborting job %d, pid=%d', job.id, job.pid)
+            logs.dbcmd('set_status', job.id, 'aborted')
+        message = {'success': 'Killing job %d' % job.id}
+        return HttpResponse(content=json.dumps(message), content_type=JSON)
+
+    message = {'error': 'PID for job %s not found' % job.id}
+    return HttpResponse(content=json.dumps(message), content_type=JSON)
+
+
 @csrf_exempt
 @cross_domain_ajax
 @require_http_methods(['POST'])
@@ -325,10 +402,16 @@ def calc_remove(request, calc_id):
         message = logs.dbcmd('del_calc', calc_id, user)
     except dbapi.NotFound:
         return HttpResponseNotFound()
-    if isinstance(message, list):  # list of removed files
+
+    if 'success' in message:
         return HttpResponse(content=json.dumps(message),
                             content_type=JSON, status=200)
-    else:  # FIXME: the error is not passed properly to the javascript
+    elif 'error' in message:
+        logging.error(message['error'])
+        return HttpResponse(content=json.dumps(message),
+                            content_type=JSON, status=403)
+    else:
+        # This is an untrapped server error
         logging.error(message)
         return HttpResponse(content=message,
                             content_type='text/plain', status=500)
@@ -403,9 +486,7 @@ def run_calc(request):
 
     user = utils.get_user_data(request)
     try:
-        job_id, fut = submit_job(einfo[0], user['name'], hazard_job_id)
-        # restart the process pool at the end of each job
-        fut .add_done_callback(lambda f: Starmap.restart())
+        job_id, pid = submit_job(einfo[0], user['name'], hazard_job_id)
     except Exception as exc:  # no job created, for instance missing .xml file
         # get the exception message
         exc_msg = str(exc)
@@ -413,22 +494,40 @@ def run_calc(request):
         response_data = exc_msg.splitlines()
         status = 500
     else:
-        response_data = dict(job_id=job_id, status='created')
+        response_data = dict(job_id=job_id, status='created', pid=pid)
         status = 200
     return HttpResponse(content=json.dumps(response_data), content_type=JSON,
                         status=status)
 
 
-def submit_job(job_ini, user_name, hazard_job_id=None,
-               loglevel=DEFAULT_LOG_LEVEL, logfile=None, exports=''):
+RUNCALC = '''\
+import os, sys
+from openquake.baselib.python3compat import pickle
+from openquake.engine import engine
+if __name__ == '__main__':
+    oqparam = pickle.loads(%(pik)r)
+    engine.run_calc(
+        %(job_id)s, oqparam, 'info', os.devnull, '', %(hazard_job_id)s)
+    os.remove(__file__)
+'''
+
+
+def submit_job(job_ini, user_name, hazard_job_id=None):
     """
     Create a job object from the given job.ini file in the job directory
-    and submit it to the job queue. Returns the job ID.
+    and run it in a new process. Returns the job ID and PID.
     """
-    job_id, oqparam = engine.job_from_file(job_ini, user_name, hazard_job_id)
-    fut = executor.submit(engine.run_calc, job_id, oqparam, loglevel,
-                          logfile, exports, hazard_job_id)
-    return job_id, fut
+    job_id, oq = engine.job_from_file(job_ini, user_name, hazard_job_id)
+    pik = pickle.dumps(oq, protocol=0)  # human readable protocol
+    code = RUNCALC % dict(job_id=job_id, hazard_job_id=hazard_job_id, pik=pik)
+    tmp_py = writetmp(code, suffix='.py')
+    # print(code, tmp_py)  # useful when debugging
+    devnull = getattr(subprocess, 'DEVNULL', None)  # defined in Python 3
+    popen = subprocess.Popen([sys.executable, tmp_py],
+                             stdin=devnull, stdout=devnull, stderr=devnull)
+    threading.Thread(target=popen.wait).start()
+    logs.dbcmd('update_job', job_id, {'pid': popen.pid})
+    return job_id, popen.pid
 
 
 @require_http_methods(['GET'])
@@ -449,7 +548,8 @@ def calc_results(request, calc_id):
     # throw back a 404.
     try:
         info = logs.dbcmd('calc_info', calc_id)
-        if user['acl_on'] and info['user_name'] != user['name']:
+        allowed_users = user['group_members'] or [user['name']]
+        if user['acl_on'] and info['user_name'] not in allowed_users:
             return HttpResponseNotFound()
     except dbapi.NotFound:
         return HttpResponseNotFound()
@@ -524,8 +624,6 @@ def get_result(request, result_id):
     try:
         job_id, job_status, datadir, ds_key = logs.dbcmd(
             'get_result', result_id)
-        if not job_status == 'complete':
-            return HttpResponseNotFound()
     except dbapi.NotFound:
         return HttpResponseNotFound()
 
@@ -563,6 +661,50 @@ def get_result(request, result_id):
     return response
 
 
+def _array(v):
+    if hasattr(v, '__toh5__'):
+        return v.__toh5__()[0]
+    return v
+
+
+@cross_domain_ajax
+@require_http_methods(['GET', 'HEAD'])
+def extract(request, calc_id, what):
+    """
+    Wrapper over the `oq extract` command. If setting.LOCKDOWN is true
+    only calculations owned by the current user can be retrieved.
+    """
+    user = utils.get_user_data(request)
+    username = user['name'] if user['acl_on'] else None
+    job = logs.dbcmd('get_job', int(calc_id), username)
+    if job is None:
+        return HttpResponseNotFound()
+
+    # read the data and save them on a temporary .pik file
+    with datastore.read(job.ds_calc_dir + '.hdf5') as ds:
+        fd, fname = tempfile.mkstemp(
+            prefix=what.replace('/', '-'), suffix='.npz')
+        os.close(fd)
+        n = len(request.path_info)
+        query_string = unquote_plus(request.get_full_path()[n:])
+        obj = _extract(ds, what + query_string)
+        if inspect.isgenerator(obj):
+            array, attrs = 0, {k: _array(v) for k, v in obj}
+        elif hasattr(obj, '__toh5__'):
+            array, attrs = obj.__toh5__()
+        else:  # assume obj is an array
+            array, attrs = obj, {}
+        numpy.savez_compressed(fname, array=array, **attrs)
+
+    # stream the data back
+    stream = FileWrapper(open(fname, 'rb'))
+    stream.close = lambda: (FileWrapper.close(stream), os.remove(fname))
+    response = FileResponse(stream, content_type='application/octet-stream')
+    response['Content-Disposition'] = (
+        'attachment; filename=%s' % os.path.basename(fname))
+    return response
+
+
 @cross_domain_ajax
 @require_http_methods(['GET'])
 def get_datastore(request, job_id):
@@ -577,9 +719,10 @@ def get_datastore(request, job_id):
         A `django.http.HttpResponse` containing the content
         of the requested artifact, if present, else throws a 404
     """
-    try:
-        job = logs.dbcmd('get_job', int(job_id), getpass.getuser())
-    except dbapi.NotFound:
+    user = utils.get_user_data(request)
+    username = user['name'] if user['acl_on'] else None
+    job = logs.dbcmd('get_job', int(job_id), username)
+    if job is None:
         return HttpResponseNotFound()
 
     fname = job.ds_calc_dir + '.hdf5'
@@ -596,9 +739,10 @@ def get_oqparam(request, job_id):
     """
     Return the calculation parameters as a JSON
     """
-    try:
-        job = logs.dbcmd('get_job', int(job_id), getpass.getuser())
-    except dbapi.NotFound:
+    user = utils.get_user_data(request)
+    username = user['name'] if user['acl_on'] else None
+    job = logs.dbcmd('get_job', int(job_id), username)
+    if job is None:
         return HttpResponseNotFound()
     with datastore.read(job.ds_calc_dir + '.hdf5') as ds:
         oq = ds['oqparam']
@@ -617,6 +761,34 @@ def web_engine_get_outputs(request, calc_id, **kwargs):
                   dict([('calc_id', calc_id)]))
 
 
+@csrf_exempt
+@cross_domain_ajax
+@require_http_methods(['POST'])
+def on_same_fs(request):
+    """
+    Accept a POST request to check access to a FS available by a client.
+
+    :param request:
+        `django.http.HttpRequest` object, containing mandatory parameters
+        filename and checksum.
+    """
+    filename = request.POST['filename']
+    checksum_in = request.POST['checksum']
+
+    checksum = 0
+    try:
+        data = open(filename, 'rb').read(32)
+        checksum = zlib.adler32(data, checksum) & 0xffffffff
+        if checksum == int(checksum_in):
+            return HttpResponse(content=json.dumps({'success': True}),
+                                content_type=JSON, status=200)
+    except (IOError, ValueError):
+        pass
+
+    return HttpResponse(content=json.dumps({'success': False}),
+                        content_type=JSON, status=200)
+
+
 @require_http_methods(['GET'])
 def license(request, **kwargs):
     return render(request, "engine/license.html")
